@@ -119,6 +119,9 @@ def get_sysmats(problem='gen_bccont', scheme=None, ppin=None,
     if problem == 'cylinderwake' or problem == 'gen_bccont':
         meshparams.update(dict(inflowvel=charvel))
 
+    if problem == 'drivencavity':
+        meshparams = dict(N=meshparams['N'])
+
     if problem == 'cylinder_rot':
         problemfem = gen_bccont_fems
         meshparams.update(dict(movingwallcntrl=True))
@@ -217,7 +220,7 @@ def get_sysmats(problem='gen_bccont', scheme=None, ppin=None,
         return femp, stokesmatsc, rhsd_vfrc, rhsd_stbc
 
 
-def drivcav_fems(N, vdgree=2, pdgree=1, scheme=None, bccontrol=None):
+def drivcav_fems(N=10, vdgree=2, pdgree=1, scheme=None, bccontrol=None):
     """dictionary for the fem items of the (unit) driven cavity
 
     Parameters
@@ -265,6 +268,11 @@ def drivcav_fems(N, vdgree=2, pdgree=1, scheme=None, bccontrol=None):
         V = dolfin.VectorFunctionSpace(mesh, "CG", vdgree)
         Q = dolfin.FunctionSpace(mesh, "CG", pdgree)
 
+    if bccontrol:
+        raise NotImplementedError()
+    else:
+        pass
+
     # Boundaries
     def top(x, on_boundary):
         return x[1] > 1.0 - dolfin.DOLFIN_EPS
@@ -993,7 +1001,7 @@ def _get_cont_shape_fun2D(xi=None, xii=None, element=None, shape='parabola'):
 
         def eval(self, value, x):
             curs = np.linalg.norm(x - xi)/lencb
-            # print(x, curs)
+            logging.debug(f'x={x}: s={curs}')
             curvel = 6*curs*(1-curs)*cbn
             value[0], value[1] = curvel[0], curvel[1]
 
