@@ -4,6 +4,8 @@ import scipy.sparse.linalg as spsla
 import time
 import logging
 
+from rich.progress import track
+
 import sadptprj_riclyap_adi.lin_alg_utils as lau
 
 # from dolfin_navier_scipy.residual_checks import get_imex_res
@@ -11,7 +13,8 @@ import sadptprj_riclyap_adi.lin_alg_utils as lau
 
 __all__ = ['cnab',
            'sbdftwo',
-           'nse_include_lnrcntrllr'
+           'nse_include_lnrcntrllr',
+           'semi_implicit_euler'
            ]
 
 
@@ -23,7 +26,8 @@ def cnab(trange=None, inivel=None, inip=None, bcs_ini=[],
          scalep=-1.,
          getbcs=None, applybcs=None, appndbcs=None,
          savevp=None, dynamic_rhs=None, dynamic_rhs_memory={},
-         check_ff=False, check_ff_maxv=None,
+         # check_ff=False,
+         check_ff_maxv=None,
          # implicit_dynamic_rhs=None, implicit_dynamic_rhs_memory={},
          ntimeslices=10, verbose=True):
     """
@@ -115,8 +119,6 @@ def cnab(trange=None, inivel=None, inip=None, bcs_ini=[],
 
             dfv_n, drm = _dynamic_rhs(ctime, vc=v_c, memory=drm,
                                       mode='abtwo')
-            # import ipdb
-            # ipdb.set_trace()
 
             rhs_n = M*v_c - .5*dt*A*v_c \
                 - (mbc_n-mbc_c) \
@@ -126,11 +128,6 @@ def cnab(trange=None, inivel=None, inip=None, bcs_ini=[],
             # cnvnrm = np.linalg.norm(3*nfc_c-nfc_o)
             # logging.info(f't:{ctime} -- rhsval:{rhsnrm}')
             # logging.info(f't:{ctime} -- cnvval:{cnvnrm}')
-            # if ctime > 0.003:
-            #     import ipdb
-            #     ipdb.set_trace()
-
-            # + .5*dt*(3*mplct_dfv_c - mplct_dfv_o) \
 
             vp_n = coeffmatlu(np.vstack([rhs_n, fp_n+bfp_n]).flatten())
 
@@ -375,7 +372,7 @@ def _onestepheun(vc=None, pc=None, tc=None, tn=None,
     NP, NV = J.shape
 
     dt = tn - tc
-    bfv_c, bfp_c, mbc_c = applybcs(bcs_c)
+    bfv_c, _, mbc_c = applybcs(bcs_c)
     fv_c = f_tdp(tc)
     nfc_c = f_vdp(appndbcs(vc, bcs_c))
     tdfv_n, drm = dynamic_rhs(tn, vc=vc, memory=drm, mode='heunpred')
@@ -408,9 +405,42 @@ def _onestepheun(vc=None, pc=None, tc=None, tn=None,
         tvp_n = lau.solve_sadpnt_smw(amat=M+.5*dt*A, jmat=J, jmatT=J.T,
                                      rhsv=tfv, rhsp=fp_n+tbfp_n)
 
+    vcpc = lau.solve_sadpnt_smw(amat=A, jmat=J, jmatT=J.T,
+                                rhsv=fv_n, rhsp=fp_n+tbfp_n)
+    nvc = vcpc[:NV, :]
+    npc = vcpc[NV:, :]
+    logging.info(f'stokes diff: {np.linalg.norm(vc-nvc)}')
+    stkres = A@vc - fv_n
+    extstkres = M@vc + dt*A@vc - tfv
+    prjstkres = lau.app_prj_via_sadpnt(rhsv=stkres, amat=A, jmat=J,
+                                       transposedprj=True)
+    logging.info(f'prjctd stokes res: {np.linalg.norm(prjstkres)}')
+    extprjstkres = lau.app_prj_via_sadpnt(rhsv=extstkres, amat=A, jmat=J,
+                                          transposedprj=True)
+    logging.info(f'extnd prjctd stokes res: {np.linalg.norm(extprjstkres)}')
+    imexres = M@nvc + dt*A@nvc + dt*J.T@npc - dt*fv_n - M@nvc
+    logging.info(f'imex res: {np.linalg.norm(imexres)}')
     tv_n = tvp_n[:NV, :]
     tp_n = 1./dt*scalep*tvp_n[NV:, :]
+    rimexres = M@tv_n + dt*A@tv_n + J.T@tvp_n[NV:, :] - dt*fv_n - M@nvc
+    logging.info(f'act imex res: {np.linalg.norm(rimexres)}')
+    rhsdiff = np.linalg.norm(tfv - dt*fv_n-M@nvc)
+    logging.info(f'rhs diff: {rhsdiff}')
+    import scipy.sparse as sps
+    bigamat = sps.vstack([sps.hstack([M+dt*A, J.T]),
+                          sps.hstack([J, sps.csr_array((NP, NP))])])
+    mvcpc = sps.linalg.spsolve(bigamat, np.vstack([tfv,
+                                                   np.zeros((NP, 1))]).flatten())
+    mvc = mvcpc[:NV].reshape((NV, 1))
+    mpc = mvcpc[NV:].reshape((NP, 1))
+    logging.info(f'diff nvc-mvc: {np.linalg.norm(nvc-mvc)}')
+    logging.info(f'diff tnv-mvc: {np.linalg.norm(tv_n-mvc)}')
+    mpcres = M@mvc + dt*A@mvc + J.T@mpc - tfv
+    logging.info(f'mpcres: {np.linalg.norm(mpcres)}')
+    # import ipdb
+    # ipdb.set_trace()
     # savevp(appndbcs(tv_new, tbcs), tp_new, time=(trange[1], 'heunpred'))
+    print(np.linalg.norm(tv_n - vc))
 
     # Corrector Step
     dfv_n, drm = dynamic_rhs(tn, vc=tv_n, memory=drm, mode='heuncorr')
@@ -525,3 +555,70 @@ def nse_include_lnrcntrllr(M=None, A=None, J=None, B=None, C=None, iniv=None,
                 getbcs=getbcsext, applybcs=applybcsext,
                 appndbcs=appndbcsext, inivel=inivext,
                 savevp=savevpext)
+
+
+def semi_implicit_euler(iniv=None, jmat=None, mmat=None, amat=None, rhsv=None,
+                        trange=None, data_trange=None, fp=None):
+    ''' integrate a NSE like system with the semi-implicit Euler method
+
+    Mv' + Av + JTp = rhs(t, v)
+          Jv       = fp
+
+
+    Parameters
+    ----------
+    rhsv : f(t, vvec) function
+        right hand side of the momentum equation
+    fp : numpy array
+        right hand side of the conti equation, optional, default as `fp=0`
+    trange : iterable
+        list or array of time instances for the integration
+    data_trange : iterable
+        list or array of time instances where to return the values
+
+    Returns
+    -------
+    ievlist : list
+        of velocity values at datatrange
+
+    Note
+    ----
+    `trange` needs to be equispaced and `data_trange` needs to be a subset
+    of `trange` and, in particular, `data_trange[0] == trange[0]`
+    '''
+
+    dtpt_trng = trange if data_trange is None else data_trange
+    ie_dtpt_trng = (np.copy(dtpt_trng)).tolist()
+    ie_dtpt_trng.pop(0)
+    (NP, NV) = jmat.shape
+    fpz = np.zeros((NP, 1)) if fp is None else fp
+    Nts = len(trange)
+
+    dt = trange[1] - trange[0]
+    # precompute a factorization of the sad point matrix
+    _, imesdpt_fctrzd = lau.solve_sadpnt_smw(amat=mmat+dt*amat, jmat=jmat,
+                                             rhsv=0*iniv, return_alu=True)
+
+    def d_impeul_increment(ct, vvec):
+        # logging.info(f'IE-int ... |vc|={np.linalg.norm(vvec)}')
+        iedfrhs = rhsv(ct, vvec)
+        # logging.info(f'IE-int ... |rhs|={np.linalg.norm(iedfrhs)}')
+        dcrhs = (mmat@vvec).reshape((-1, 1)) + dt*iedfrhs
+        dslvdrhs = imesdpt_fctrzd(np.vstack([dcrhs, fpz]))
+        # logging.info(f'IE-int ... |v+|={np.linalg.norm(dslvdrhs[:NV])}')
+        return dslvdrhs[:NV]
+
+    ievlist = [iniv]
+    cvn = iniv
+    logging.info(f'Impl. Euler integration with {Nts} time steps')
+    for ct in track(trange[1:], description='semi-IE ongoing'):
+        # logging.info(f'IE-int ... time={ct}:')
+        cvp = cvn
+        # ievlist.append(cv + impeul_increment(cv))
+        cvn = d_impeul_increment(ct, cvp)
+        if ct == ie_dtpt_trng[0]:
+            ievlist.append(cvn)
+            ie_dtpt_trng.pop(0)
+        else:
+            pass  # only record at data points
+    return ievlist
